#pragma semicolon 1
#include <sourcemod>
#include <sdktools>

#define PL_VERSION "1.0"

new Handle:cvar_alltalk = INVALID_HANDLE;

new Handle:g_Enabled;
new Handle:g_Msgs = INVALID_HANDLE;
new Handle:g_Notify = INVALID_HANDLE;
new Handle:g_delayTime = INVALID_HANDLE;
new Handle:g_Sounds = INVALID_HANDLE;
new Handle:g_OnSound = INVALID_HANDLE;
new Handle:g_OffSound = INVALID_HANDLE;
new Handle:g_MinPlayers = INVALID_HANDLE;
new bool:delayedchange = false;

// Soundfile
new String:g_OnSnd[255]	= "weapons/ar2/ar2_reload_push.wav";
new String:g_OffSnd[255] = "weapons/ar2/ar2_reload_rotate.wav";

public Plugin:myinfo = {
	name        = "Dod:Source Alltalkmanager",
	author      = "BenSib",
	description = "Automatically manages your server's alltalk-settings",
	version     = PL_VERSION,
	url         = "http://www.dodsplugins.com"
}

public OnPluginStart()
{
	CreateConVar("dod_alltalkmanager_version", PL_VERSION, "DoD:S Alltalkmanager", FCVAR_DONTRECORD|FCVAR_PLUGIN|FCVAR_SPONLY|FCVAR_REPLICATED|FCVAR_NOTIFY);
	
	cvar_alltalk = FindConVar("sv_alltalk");
	SetConVarFlags(cvar_alltalk, GetConVarFlags(cvar_alltalk)|FCVAR_NOTIFY);
	
	g_Enabled  = CreateConVar("dod_alltalkmanager_enabled", "1",  "Enable/disable Alltalkmanager.",FCVAR_PLUGIN);
	g_Msgs = CreateConVar("dod_alltalkmanager_changemessage", "1", "<0/1> = enable/disable displaying Messages when alltalk-settings have been changed <default: 1>", FCVAR_PLUGIN, true, 0.0, true, 1.0);
	g_Notify = CreateConVar("dod_alltalkmanager_spawnmessage", "1", "<0/1/2> = 0: no spawnnotification / 1: notify about alltalk only during delaytime / 2: always notify about alltalk <default: 1>", FCVAR_PLUGIN, true, 0.0, true, 2.0);
	g_delayTime = CreateConVar("dod_alltalkmanager_delaytime", "25", "<#> = number of seconds since roundwin alltalk stays enabled <default: 25>", FCVAR_PLUGIN, true, 5.0, true, 120.0);
	g_Sounds = CreateConVar("dod_alltalkmanager_enablesounds", "0", "<0/1> = enable/disable Sounds when alltalk-settings have been changed <default: 0>", FCVAR_PLUGIN, true, 0.0, true, 1.0);
	g_OnSound = CreateConVar("dod_alltalkmanager_onsound", g_OnSnd, "when sounds are enabled insert path to soundfile <default: weapons/ar2/ar2_reload_push.wav>", FCVAR_PLUGIN);
	g_OffSound = CreateConVar("dod_alltalkmanager_offsound", g_OffSnd, "when sounds are enabled insert path to soundfile <default: weapons/ar2/ar2_reload_rotate.wav>", FCVAR_PLUGIN);
	g_MinPlayers = CreateConVar("dod_alltalkmanager_minplayers", "12", "Required minimum number of players to turn alltalk off <default: 12>", FCVAR_PLUGIN);
	
	RegConsoleCmd("say",         Command_Say);
	RegConsoleCmd("say_team",    Command_Say);
	
	HookEventEx("dod_round_win", OnDoDRoundWin, EventHookMode_Post);
	HookEventEx("player_spawn",PlayerSpawn,EventHookMode_Pre);
		
	AutoExecConfig(true,"dod_alltalkmanager", "dod_alltalkmanager");
}

public OnMapStart()
{
	if(GetConVarBool(g_Enabled))
	{
		SetConVarBool(cvar_alltalk, true);
		if (GetConVarBool(g_Sounds))
		{
			GetConVarString(g_OnSound, g_OnSnd, sizeof(g_OnSnd));
			GetConVarString(g_OffSound, g_OffSnd, sizeof(g_OffSnd));
			PrecacheSound(g_OnSnd, true);
			PrecacheSound(g_OffSnd, true);
		}
	}
}

public PlayerSpawn(Handle:event,const String:name[],bool:dontBroadcast)
{
	if(GetConVarBool(g_Enabled))
	{
		new client = GetClientOfUserId(GetEventInt(event, "userid"));
		if (GetConVarBool(cvar_alltalk))
		{
			if (delayedchange)
			{
				//Alltalk will be turned on, no check needed
				if (GetConVarInt(g_Notify) == 1)
					PrintToChat(client, "\x01[\x05Alltalk\x01 is still \x04ON\x01]");
				else if (GetConVarInt(g_Notify) == 2)
					PrintToChat(client, "\x01[\x05Alltalk\x01 is \x04ON\x01]");
			}
			else
			{
				//Check if alltalk can be turned off immediately - it will then stay off until Round Ends !
				if ((GetTeamClientCount(2) + GetTeamClientCount(3)) >= GetConVarInt(g_MinPlayers))
					DisableAlltalk();
				else if (GetConVarInt(g_Notify) == 2)
					PrintToChat(client, "\x01[\x05Alltalk\x01 is \x04ON\x01]");
			}
		}
		else
		{
			if (GetConVarInt(g_Notify) == 2)
				PrintToChat(client, "\x01[\x05Alltalk\x01 is \x04OFF\x01]");
		}
	}
}

public Action:OnDoDRoundWin(Handle:event, const String:name[], bool:dontBroadcast)
{
	if(GetConVarBool(g_Enabled))
	{
		if (!GetConVarBool(cvar_alltalk))
			EnableAlltalk();
		if ((GetTeamClientCount(2) + GetTeamClientCount(3)) >= GetConVarInt(g_MinPlayers))
		{
			delayedchange = true;
			CreateTimer(GetConVarFloat(g_delayTime), DisableAlltalktimer, _, TIMER_FLAG_NO_MAPCHANGE);
		}
		else 
			delayedchange = false;
	}
	return Plugin_Continue;
}

public EnableAlltalk()
{	
	SetConVarBool(cvar_alltalk, true);
	if (GetConVarBool(g_Sounds))
		EmitSoundToAll(g_OnSnd, _, _, _, _, 1.0);
	if (GetConVarBool(g_Msgs))
	{
		PrintHintTextToAll("Alltalk is now ON");
		PrintToChatAll("\x01[\x05Alltalk\x01 is now \x04ON\x01]");
	}
}

public DisableAlltalk()
{	
	SetConVarBool(cvar_alltalk, false);
	if (GetConVarBool(g_Sounds))
		EmitSoundToAll(g_OffSnd, _, _, _, _, 1.0);
	if (GetConVarBool(g_Msgs))
	{
		PrintHintTextToAll("Alltalk is now OFF");
		PrintToChatAll("\x01[\x05Alltalk\x01 is now \x04OFF\x01]");
	}
}

public Action:DisableAlltalktimer(Handle:timer)
{	
	delayedchange = false;
	DisableAlltalk();
}

public Action:Command_Say(client, args) 
{
	new iStart = 0, String:sCommand[64];
	GetCmdArgString(sCommand, sizeof(sCommand));
	
	if (sCommand[strlen(sCommand) - 1] == '"') {
		iStart++;
		sCommand[strlen(sCommand) - 1] = '\0';
	}
	
	if(StrEqual(sCommand[iStart], "!alltalk")) 
	{
		if (GetConVarBool(cvar_alltalk))
		{
			PrintToChat(client, "\x01[\x05Alltalk \x01is \x04ON\x01]");
		}
		else
		{
			PrintToChat(client, "\x01[\x05Alltalk \x01is \x04OFF\x01]");
		}
	}
}

